@@ -30,6 +30,7 @@ import (
 	"testing"
 
 	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/apimachinery/pkg/types"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
@@ -38,6 +39,7 @@ import (
 	triggersv1 "github.com/tektoncd/triggers/pkg/apis/triggers/v1alpha1"
 	dynamicclientset "github.com/tektoncd/triggers/pkg/client/dynamic/clientset"
 	"github.com/tektoncd/triggers/pkg/client/dynamic/clientset/tekton"
+	interceptorinformer "github.com/tektoncd/triggers/pkg/client/injection/informers/triggers/v1alpha1/clusterinterceptor"
 	clustertriggerbindinginformer "github.com/tektoncd/triggers/pkg/client/injection/informers/triggers/v1alpha1/clustertriggerbinding"
 	eventlistenerinformer "github.com/tektoncd/triggers/pkg/client/injection/informers/triggers/v1alpha1/eventlistener"
 	triggerinformer "github.com/tektoncd/triggers/pkg/client/injection/informers/triggers/v1alpha1/trigger"
@@ -61,6 +63,7 @@ import (
 	fakedynamic "k8s.io/client-go/dynamic/fake"
 	"k8s.io/client-go/kubernetes"
 	ktesting "k8s.io/client-go/testing"
+	"knative.dev/pkg/apis"
 	"knative.dev/pkg/ptr"
 	rtesting "knative.dev/pkg/reconciler/testing"
 )
@@ -68,6 +71,7 @@ import (
 const (
 	eventID   = "12345"
 	namespace = "foo"
+	elUID     = "el-uid"
 )
 
 func init() {
@@ -75,6 +79,51 @@ func init() {
 	template.UUID = func() string { return eventID }
 }
 
+var (
+	github = &triggersv1.ClusterInterceptor{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "github",
+		},
+		Spec: triggersv1.ClusterInterceptorSpec{
+			ClientConfig: triggersv1.ClientConfig{
+				URL: &apis.URL{
+					Scheme: "http",
+					Host:   "tekton-triggers-core-interceptors",
+					Path:   "/github",
+				},
+			},
+		},
+	}
+	cel = &triggersv1.ClusterInterceptor{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "cel",
+		},
+		Spec: triggersv1.ClusterInterceptorSpec{
+			ClientConfig: triggersv1.ClientConfig{
+				URL: &apis.URL{
+					Scheme: "http",
+					Host:   "tekton-triggers-core-interceptors",
+					Path:   "/cel",
+				},
+			},
+		},
+	}
+	bitbucket = &triggersv1.ClusterInterceptor{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "bitbucket",
+		},
+		Spec: triggersv1.ClusterInterceptorSpec{
+			ClientConfig: triggersv1.ClientConfig{
+				URL: &apis.URL{
+					Scheme: "http",
+					Host:   "tekton-triggers-core-interceptors",
+					Path:   "/bitbucket",
+				},
+			},
+		},
+	}
+)
+
 // getSinkAssets seeds test resources and returns a testable Sink and a dynamic client. The returned client is used to
 // create the fake resources and can be used to check if the correct resources were created.
 func getSinkAssets(t *testing.T, resources test.Resources, elName string, webhookInterceptor http.Handler) (Sink, *fakedynamic.FakeDynamicClient) {
@@ -90,6 +139,7 @@ func getSinkAssets(t *testing.T, resources test.Resources, elName string, webhoo
 	// Setup a handler for core interceptors using httptest
 	httpClient := setupInterceptors(t, clients.Kube, logger.Sugar(), webhookInterceptor)
 
+	recorder, _ := NewRecorder()
 	r := Sink{
 		EventListenerName:           elName,
 		EventListenerNamespace:      namespace,
@@ -100,11 +150,13 @@ func getSinkAssets(t *testing.T, resources test.Resources, elName string, webhoo
 		HTTPClient:                  httpClient,
 		Logger:                      logger.Sugar(),
 		Auth:                        DefaultAuthOverride{},
+		Recorder:                    recorder,
 		EventListenerLister:         eventlistenerinformer.Get(ctx).Lister(),
 		TriggerLister:               triggerinformer.Get(ctx).Lister(),
 		TriggerBindingLister:        triggerbindinginformer.Get(ctx).Lister(),
 		ClusterTriggerBindingLister: clustertriggerbindinginformer.Get(ctx).Lister(),
 		TriggerTemplateLister:       triggertemplateinformer.Get(ctx).Lister(),
+		ClusterInterceptorLister:    interceptorinformer.Get(ctx).Lister(),
 	}
 	return r, dynamicClient
 }
@@ -171,9 +223,10 @@ func checkSinkResponse(t *testing.T, resp *http.Response, elName string) {
 		t.Fatalf("Error reading response body: %s", err)
 	}
 	wantBody := Response{
-		EventListener: elName,
-		Namespace:     namespace,
-		EventID:       eventID,
+		EventListener:    elName,
+		EventListenerUID: elUID,
+		Namespace:        namespace,
+		EventID:          eventID,
 	}
 	if diff := cmp.Diff(wantBody, gotBody); diff != "" {
 		t.Errorf("did not get expected response back -want,+got: %s", diff)
@@ -339,6 +392,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					Triggers: []triggersv1.EventListenerTrigger{{
@@ -402,6 +456,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					NamespaceSelector: triggersv1.NamespaceSelector{
@@ -412,6 +467,151 @@ func TestHandleEvent(t *testing.T) {
 		},
 		eventBody: eventBody,
 		want:      []pipelinev1.TaskRun{gitCloneTaskRun},
+	}, {
+		name: "label selector match expressions",
+		resources: test.Resources{
+			Namespaces: []*corev1.Namespace{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "bar",
+				},
+			}},
+			TriggerBindings: []*triggersv1.TriggerBinding{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "git-clone",
+					Namespace: "bar",
+				},
+				Spec: triggersv1.TriggerBindingSpec{
+					Params: []triggersv1.Param{
+						{Name: "url", Value: "$(body.repository.url)"},
+						{Name: "revision", Value: "$(body.head_commit.id)"},
+						{Name: "name", Value: "git-clone-run"},
+						{Name: "app", Value: "$(body.foo)"},
+						{Name: "type", Value: "$(header.Content-Type)"},
+					},
+				},
+			}},
+			TriggerTemplates: []*triggersv1.TriggerTemplate{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "git-clone",
+					Namespace: "bar",
+				},
+				Spec: gitCloneTTSpec,
+			}},
+			Triggers: []*triggersv1.Trigger{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "git-clone-trigger",
+					Namespace: "bar",
+					Labels: map[string]string{
+						"foo": "bar",
+					},
+				},
+				Spec: triggersv1.TriggerSpec{
+					Bindings: []*triggersv1.TriggerSpecBinding{{Ref: "git-clone"}},
+					Template: triggersv1.TriggerSpecTemplate{Ref: ptr.String("git-clone")},
+				},
+			}, {
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "second-not-invoked-trigger",
+					Namespace: "bar",
+					Labels: map[string]string{
+						"foo": "notbar",
+					},
+				},
+				Spec: triggersv1.TriggerSpec{
+					Bindings: []*triggersv1.TriggerSpecBinding{{Ref: "git-clone"}},
+					Template: triggersv1.TriggerSpecTemplate{Ref: ptr.String("git-clone")},
+				},
+			}},
+			EventListeners: []*triggersv1.EventListener{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      eventListenerName,
+					Namespace: namespace,
+					UID:       types.UID(elUID),
+				},
+				Spec: triggersv1.EventListenerSpec{
+					NamespaceSelector: triggersv1.NamespaceSelector{
+						MatchNames: []string{"bar"},
+					},
+					LabelSelector: &metav1.LabelSelector{
+						MatchLabels: map[string]string{"foo": "bar"},
+					},
+				},
+			}},
+		},
+		eventBody: eventBody,
+		// only one of the tasks is invoked
+		want: []pipelinev1.TaskRun{gitCloneTaskRun},
+	}, {
+		name: "label selector match expressions without namespace selector",
+		resources: test.Resources{
+			Namespaces: []*corev1.Namespace{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "bar",
+				},
+			}},
+			TriggerBindings: []*triggersv1.TriggerBinding{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "git-clone",
+					Namespace: namespace,
+				},
+				Spec: triggersv1.TriggerBindingSpec{
+					Params: []triggersv1.Param{
+						{Name: "url", Value: "$(body.repository.url)"},
+						{Name: "revision", Value: "$(body.head_commit.id)"},
+						{Name: "name", Value: "git-clone-run"},
+						{Name: "app", Value: "$(body.foo)"},
+						{Name: "type", Value: "$(header.Content-Type)"},
+					},
+				},
+			}},
+			TriggerTemplates: []*triggersv1.TriggerTemplate{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "git-clone",
+					Namespace: namespace,
+				},
+				Spec: gitCloneTTSpec,
+			}},
+			Triggers: []*triggersv1.Trigger{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "git-clone-trigger",
+					Namespace: namespace,
+					Labels: map[string]string{
+						"foo": "bar",
+					},
+				},
+				Spec: triggersv1.TriggerSpec{
+					Bindings: []*triggersv1.TriggerSpecBinding{{Ref: "git-clone"}},
+					Template: triggersv1.TriggerSpecTemplate{Ref: ptr.String("git-clone")},
+				},
+			}, {
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "second-not-invoked-trigger",
+					Namespace: namespace,
+					Labels: map[string]string{
+						"foo": "notbar",
+					},
+				},
+				Spec: triggersv1.TriggerSpec{
+					Bindings: []*triggersv1.TriggerSpecBinding{{Ref: "git-clone"}},
+					Template: triggersv1.TriggerSpecTemplate{Ref: ptr.String("git-clone")},
+				},
+			}},
+			EventListeners: []*triggersv1.EventListener{{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      eventListenerName,
+					Namespace: namespace,
+					UID:       types.UID(elUID),
+				},
+				Spec: triggersv1.EventListenerSpec{
+					LabelSelector: &metav1.LabelSelector{
+						MatchLabels: map[string]string{"foo": "bar"},
+					},
+				},
+			}},
+		},
+		eventBody: eventBody,
+		// only one of the tasks is invoked
+		want: []pipelinev1.TaskRun{gitCloneTaskRun},
 	}, {
 		name: "eventlistener with a trigger ref",
 		resources: test.Resources{
@@ -431,6 +631,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					Triggers: []triggersv1.EventListenerTrigger{{
@@ -458,6 +659,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					Triggers: []triggersv1.EventListenerTrigger{{
@@ -480,6 +682,7 @@ func TestHandleEvent(t *testing.T) {
 					"secretKey": []byte("secret"),
 				},
 			}},
+			ClusterInterceptors: []*triggersv1.ClusterInterceptor{github, cel},
 			Triggers: []*triggersv1.Trigger{{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "git-clone-trigger",
@@ -487,15 +690,19 @@ func TestHandleEvent(t *testing.T) {
 				},
 				Spec: triggersv1.TriggerSpec{
 					Interceptors: []*triggersv1.EventInterceptor{{
-						GitHub: &triggersv1.GitHubInterceptor{
-							SecretRef: &triggersv1.SecretRef{
+						Ref: triggersv1.InterceptorRef{Name: "github"},
+						Params: []triggersv1.InterceptorParams{{
+							Name: "secretRef",
+							Value: test.ToV1JSON(t, &triggersv1.SecretRef{
 								SecretKey:  "secretKey",
 								SecretName: "secret",
-							},
-							EventTypes: []string{"pull_request"},
-						},
+							}),
+						}, {
+							Name:  "eventTypes",
+							Value: test.ToV1JSON(t, []string{"pull_request"}),
+						}},
 					}, {
-						CEL: &triggersv1.CELInterceptor{
+						DeprecatedCEL: &triggersv1.CELInterceptor{
 							Overlays: []triggersv1.CELOverlay{
 								// FIXME: We aren't really testing that this value can be used in a binding tho
 								{Key: "new", Expression: "body.repository.url"},
@@ -510,6 +717,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					Triggers: []triggersv1.EventListenerTrigger{{
@@ -536,6 +744,7 @@ func TestHandleEvent(t *testing.T) {
 					"secretKey": []byte("secret"),
 				},
 			}},
+			ClusterInterceptors: []*triggersv1.ClusterInterceptor{bitbucket},
 			Triggers: []*triggersv1.Trigger{{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "git-clone-trigger",
@@ -543,13 +752,17 @@ func TestHandleEvent(t *testing.T) {
 				},
 				Spec: triggersv1.TriggerSpec{
 					Interceptors: []*triggersv1.EventInterceptor{{
-						Bitbucket: &triggersv1.BitbucketInterceptor{
-							SecretRef: &triggersv1.SecretRef{
+						Ref: triggersv1.InterceptorRef{Name: "bitbucket"},
+						Params: []triggersv1.InterceptorParams{{
+							Name: "secretRef",
+							Value: test.ToV1JSON(t, &triggersv1.SecretRef{
 								SecretKey:  "secretKey",
 								SecretName: "secret",
-							},
-							EventTypes: []string{"repo:refs_changed"},
-						},
+							}),
+						}, {
+							Name:  "eventTypes",
+							Value: test.ToV1JSON(t, []string{"repo:refs_changed"}),
+						}},
 					}},
 					Bindings: gitCloneTBSpec,
 					Template: triggersv1.TriggerSpecTemplate{Spec: &gitCloneTTSpec},
@@ -559,6 +772,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					Triggers: []triggersv1.EventListenerTrigger{{
@@ -580,6 +794,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					NamespaceSelector: triggersv1.NamespaceSelector{
@@ -628,6 +843,7 @@ func TestHandleEvent(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      eventListenerName,
 					Namespace: namespace,
+					UID:       types.UID(elUID),
 				},
 				Spec: triggersv1.EventListenerSpec{
 					Triggers: []triggersv1.EventListenerTrigger{{
@@ -686,7 +902,9 @@ func TestHandleEvent(t *testing.T) {
 			// TODO: Do we ever support multiple eventListeners? Maybe change test.Resources to only accept one?
 			elName := tc.resources.EventListeners[0].Name
 			sink, dynamicClient := getSinkAssets(t, tc.resources, elName, tc.webhookInterceptor)
-			ts := httptest.NewServer(http.HandlerFunc(sink.HandleEvent))
+
+			metricsRecorder := &MetricsHandler{Handler: http.HandlerFunc(sink.HandleEvent)}
+			ts := httptest.NewServer(http.HandlerFunc(metricsRecorder.Intercept(sink.NewMetricsRecorderInterceptor())))
 			defer ts.Close()
 			req, err := http.NewRequest("POST", ts.URL, bytes.NewReader(tc.eventBody))
 			if err != nil {
@@ -769,13 +987,17 @@ func TestHandleEvent_AuthOverride(t *testing.T) {
 				Spec: triggersv1.TriggerSpec{
 					ServiceAccountName: testCase.userVal,
 					Interceptors: []*triggersv1.EventInterceptor{{
-						GitHub: &triggersv1.GitHubInterceptor{
-							SecretRef: &triggersv1.SecretRef{
+						Ref: triggersv1.InterceptorRef{Name: "github"},
+						Params: []triggersv1.InterceptorParams{{
+							Name: "secretRef",
+							Value: test.ToV1JSON(t, &triggersv1.SecretRef{
 								SecretKey:  "secretKey",
 								SecretName: "secret",
-							},
-							EventTypes: []string{"pull_request"},
-						},
+							}),
+						}, {
+							Name:  "eventTypes",
+							Value: test.ToV1JSON(t, []string{"pull_request"}),
+						}},
 					}},
 					Bindings: []*triggersv1.TriggerSpecBinding{{
 						Name:  "url",
@@ -840,12 +1062,12 @@ func TestHandleEvent_AuthOverride(t *testing.T) {
 					"secretKey": []byte("secret"),
 				},
 			}
-
 			resources := test.Resources{
-				Triggers:        []*triggersv1.Trigger{trigger},
-				EventListeners:  []*triggersv1.EventListener{el},
-				Secrets:         []*corev1.Secret{secret, authSecret},
-				ServiceAccounts: []*corev1.ServiceAccount{authSA},
+				ClusterInterceptors: []*triggersv1.ClusterInterceptor{github},
+				Triggers:            []*triggersv1.Trigger{trigger},
+				EventListeners:      []*triggersv1.EventListener{el},
+				Secrets:             []*corev1.Secret{secret, authSecret},
+				ServiceAccounts:     []*corev1.ServiceAccount{authSA},
 			}
 			sink, dynamicClient := getSinkAssets(t, resources, el.Name, nil)
 			sink.Auth = fakeAuth{}
@@ -1105,13 +1327,18 @@ func TestExecuteInterceptor_error(t *testing.T) {
 }
 
 func TestExecuteInterceptor_NotContinue(t *testing.T) {
-	s, _ := getSinkAssets(t, test.Resources{}, "el-name", nil)
+	resources := test.Resources{
+		ClusterInterceptors: []*triggersv1.ClusterInterceptor{cel},
+	}
+	s, _ := getSinkAssets(t, resources, "el-name", nil)
 	trigger := triggersv1.Trigger{
 		Spec: triggersv1.TriggerSpec{
 			Interceptors: []*triggersv1.EventInterceptor{{
-				CEL: &triggersv1.CELInterceptor{
-					Filter: `body.head == "abcde"`,
-				},
+				Ref: triggersv1.InterceptorRef{Name: "cel"},
+				Params: []triggersv1.InterceptorParams{{
+					Name:  "filter",
+					Value: test.ToV1JSON(t, `body.head == "abcde"`),
+				}},
 			}}},
 	}
 	url, _ := url.Parse("http://example.com")
@@ -1149,29 +1376,26 @@ func (f *echoInterceptor) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 }
 
 func TestExecuteInterceptor_ExtensionChaining(t *testing.T) {
-	ctx, _ := rtesting.SetupFakeContext(t)
-	clients := test.SeedResources(t, ctx, test.Resources{})
-	logger := zaptest.NewLogger(t).Sugar()
-
 	webhookInterceptorName := "foo"
 	echoServer := &echoInterceptor{}
-	httpClient := setupInterceptors(t, clients.Kube, logger, echoServer)
-	s := Sink{
-		HTTPClient: httpClient,
-		Logger:     logger,
+	resources := test.Resources{
+		ClusterInterceptors: []*triggersv1.ClusterInterceptor{cel},
 	}
+	s, _ := getSinkAssets(t, resources, "", echoServer)
 
 	sha := "abcdefghi" // Fake "sha" to send via body
 	// trigger has a chain of 3 interceptors CEL(overlay) -> webhook -> CEL(filter)
 	trigger := triggersv1.Trigger{
 		Spec: triggersv1.TriggerSpec{
 			Interceptors: []*triggersv1.EventInterceptor{{
-				CEL: &triggersv1.CELInterceptor{
-					Overlays: []triggersv1.CELOverlay{{
+				Ref: triggersv1.InterceptorRef{Name: "cel"},
+				Params: []triggersv1.InterceptorParams{{
+					Name: "overlays",
+					Value: test.ToV1JSON(t, []triggersv1.CELOverlay{{
 						Key:        "truncated_sha",
 						Expression: "body.sha.truncate(5)",
-					}},
-				},
+					}}),
+				}},
 			}, {
 				Webhook: &triggersv1.WebhookInterceptor{
 					ObjectRef: &corev1.ObjectReference{
@@ -1181,9 +1405,11 @@ func TestExecuteInterceptor_ExtensionChaining(t *testing.T) {
 					},
 				},
 			}, {
-				CEL: &triggersv1.CELInterceptor{
-					Filter: "body.extensions.truncated_sha == \"abcde\" && extensions.truncated_sha == \"abcde\"",
-				},
+				Ref: triggersv1.InterceptorRef{Name: "cel"},
+				Params: []triggersv1.InterceptorParams{{
+					Name:  "filter",
+					Value: test.ToV1JSON(t, "body.extensions.truncated_sha == \"abcde\" && extensions.truncated_sha == \"abcde\""),
+				}},
 			}},
 		},
 	}
@@ -1193,7 +1419,7 @@ func TestExecuteInterceptor_ExtensionChaining(t *testing.T) {
 		t.Fatalf("http.NewRequest: %v", err)
 	}
 	body := fmt.Sprintf(`{"sha": "%s"}`, sha)
-	resp, _, iresp, err := s.ExecuteInterceptors(trigger, req, []byte(body), logger, eventID)
+	resp, _, iresp, err := s.ExecuteInterceptors(trigger, req, []byte(body), s.Logger, eventID)
 	if err != nil {
 		t.Fatalf("executeInterceptors: %v", err)
 	}
